C-----------------------------------------------------------------------
      SUBROUTINE ephinitu (unit,ierr)
C
C  Read cpf ephemeris file into memory.
C
C  Input
C	unit	- fortran unit number
C
C  Output
C	ierr	- error code: 0= OK, -1= error
C	contents of /ephihdr/ (header info)
C	contents of /ephi/ (record info)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date: 1 May 2003
C  History:
C  06-Oct-2006	- Correct reading of leapflag. rlr. v1.01a
C  02/09/2015 - Record the first entry where leap second flag is non-zero. 
C               Leapsec_time used in ephinterp.c to determine times to correct 
C               requested interpolation time back to pre-leap second time. rlr.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (a-h,o-z)

      INCLUDE  '../include/ephihdr.inc'
      INCLUDE  '../include/ephi.inc'

      COMMON /leap/ leapsec_itime, leapsec_value
      DOUBLE PRECISION leapsec_itime    ! =-1 if no leap second found;
                                        ! =time into cpf  of leap second
      INTEGER leapsec_value             ! value of leap second at leapsec_itime

      COMMON /deb/ debug
      LOGICAL debug

      INTEGER idir
      INTEGER mjd
      INTEGER unit, ierr
      INTEGER i,j
      INTEGER*4 leapflg
      DOUBLE PRECISION dgast
      DOUBLE PRECISION sod
      DOUBLE PRECISION vec(3), corr
      DOUBLE PRECISION pi
      CHARACTER*100 ephstr

C     initialize all the arrays, in case there is nothing to read */
      DO i= 1, MAXENT
        DO j= 1, 6
          outv(i,j)= 0.d0
          backv(i,j)= 0.d0
        ENDDO
        DO j= 1, 3
          aberoutv(i,j)= 0.d0
          aberbackv(i,j)= 0.d0
          offsetoutv(i,j)= 0.d0
          offsetbackv(i,j)= 0.d0
          rotangv(i,j)= 0.d0
        ENDDO
        c12(i)= 0.d0
        c23(i)= 0.d0
        leapflag(i)= 0
        oscrelcorr(i)= 0.d0
        gast(i)= 0.d0
        eopxy(i,1)= 0.d0
        eopxy(i,2)= 0.d0
        eopdutc(i)= 0.d0
      ENDDO

C  Rarely-used header parameters. Make 'em transparent.
      atrk_ro_0= 0
      xtrk_ro_0= 0
      rtrk_ro_0= 0
      atrk_ro_1= 0
      xtrk_ro_1= 0
      rtrk_ro_1= 0
      atrk_ro_2= 0
      xtrk_ro_2= 0
      rtrk_ro_2= 0
      prf= 0.d0
      xpond_xmit_delay= 0.d0
      xpond_utc_off= 0.d0
      xpond_osc_drift= 0.d0
      cofm_corr= 0.d0
  
      nv=0
      nvoff= 0
      nvrot= 0
      dgast= 0.d0
      nveop= 0
      ierr= 0
      leapsec_itime= -1.0d0
      leapsec_value= 0

      pi=4.D0*datan(1.D0)

   10 READ (unit,20,end=120) ephstr
CC	if (ephstr(1:1).eq.'H') write (*,20)ephstr
CC	write (*,20)ephstr
   20 FORMAT (a100)
CC      write(*,*) '[',ephstr,']'


C  Headers
      IF (ephstr(1:2).eq.'h1' .OR. ephstr(1:2).eq.'H1') THEN
        READ (ephstr,30,err=100) version, eph_source,
     &     epyear,epmon,epday,ephour,eph_seq,tar_name,eph_notes
        if (debug) WRITE (*,*) eph_source,':',eph_notes
   30   FORMAT (7x,i2,2x,a3,1x,i4,1x,i2,1x,i2,1x,i2,2x,i4,1x,a10,1x,a10)


      ELSE IF (ephstr(1:2).eq.'h2' .OR. ephstr(1:2).eq.'H2') THEN
        READ (ephstr,40,err=100) cospar_id,sic,norad,
     &    sttpyear,sttpmonth,sttpday,sttphour,sttpmin,sttpsec,
     &    endpyear,endpmonth,endpday,endphour,endpmin,endpsec,
     &    ephsep,compat,tar_type,ref_frame,rot_type,cofm_app
   40   FORMAT (3x,i8,1x,i4,1x,i8,1x,2(i4,5(1x,i2),1x),i5,1x,i1,
     &   1x,i1,1x,i2, 1x,i1, 1x,i1)


      ELSE IF (ephstr(1:2).eq.'h3' .OR. ephstr(1:2).eq.'H3') THEN
        READ (ephstr,50,err=100) atrk_ro_0,xtrk_ro_0,rtrk_ro_0,
     x   atrk_ro_1,xtrk_ro_1,rtrk_ro_1,atrk_ro_2,xtrk_ro_2,rtrk_ro_2
   50   FORMAT (2x,9(1x,i5))


      ELSE IF (ephstr(1:2).eq.'h4' .OR. ephstr(1:2).eq.'H4') THEN
        READ (ephstr,60,err=100) prf,xpond_xmit_delay,xpond_utc_off,
     x   xpond_osc_drift
   60   FORMAT (3x,f12.5,1x,f10.4,2(1x,f11.2))


      ELSE IF (ephstr(1:2).eq.'h5' .OR. ephstr(1:2).eq.'H5') THEN
        READ (ephstr,70,err=100) cofm_corr
   70   FORMAT (3x,f7.4)


      ELSE IF (ephstr(1:2).eq.'h9' .OR. ephstr(1:2).eq.'H9') THEN


C  Individual time records

C  Positions
      ELSE IF (ephstr(1:2).eq.'10' ) THEN
        IF (nv.gt.MAXENT) THEN
          WRITE (*,*) 'ephemeris read error: too many records:',nv
          GO TO 110
        END IF
        READ (ephstr(3:100),*,err=100) idir,mjd,sod,leapflg,
     &       (vec(k),k=1,3)

        IF (idir .eq. 0 .OR. idir .eq. 1) nv=nv+1
        leapflag(nv)=leapflg
        jdi(nv)=mjd+2400000.0d0
        jdf(nv)=sod/86400.d0+0.5d0
        IF (jdf(nv).ge.1.0d0) THEN
          jdi(nv)=jdi(nv)+1.0d0
          jdf(nv)=jdf(nv)-1.0d0
        END IF

C  ...Mark start of leap sec flag being non-zero */
        ! set to 0h UTC of first day of non-zero leapsecond flag
        if (leapflag(nv) .ne. 0 .and. leapsec_itime .le. -1.d-6) then
          leapsec_itime= (jdi(nv)- jdi(1) +
     &                   (0.5d0- jdf(1))) * 86400.d0;
          leapsec_value= leapflag(nv);
        endif
CC        write (*,*) "leap sec",leapsec_itime, " value: ",leapsec_value

C    Outbound
        IF (idir .eq. 0 .OR. idir .eq. 1) THEN
C   ...Assume idir cannot be 0 and 1 in same file
          ltcorr= idir;
          outv(nv,1)= vec(1)
          outv(nv,2)= vec(2)
          outv(nv,3)= vec(3)

C   ...In case there is not back vector (e.g., satellites)...
          backv(nv,1)= -outv(nv,1)
          backv(nv,2)= -outv(nv,2)
          backv(nv,3)= -outv(nv,3)

C    Inbound
        ELSE IF (idir .eq. 2) THEN
          backv(nv,1)= vec(1)
          backv(nv,2)= vec(2)
          backv(nv,3)= vec(3)

C   ...Be sure out and back are opposite directions.
          IF (backv(nv,1)*outv(nv,1) .gt. 0.d0) THEN
            backv(nv,1)= -backv(nv,1)
            backv(nv,2)= -backv(nv,2)
            backv(nv,3)= -backv(nv,3)
          ENDIF
        ENDIF

C  Velocities
      ELSE IF (ephstr(1:2).eq.'20') THEN
        READ (ephstr(3:100),*,err=100) idir, (vec(k),k=1,3)

C    Outbound
        IF (idir .eq. 0 .OR. idir .eq. 1) THEN
          outv(nv,4)= vec(1)
          outv(nv,5)= vec(2)
          outv(nv,6)= vec(3)

C   ...In case there is not back vector (e.g., satellites)...
          backv(nv,4)= -outv(nv,4)
          backv(nv,5)= -outv(nv,5)
          backv(nv,6)= -outv(nv,6)

C    Inbound
        ELSE IF (idir .eq. 2) THEN
          backv(nv,4)= vec(1)
          backv(nv,5)= vec(2)
          backv(nv,6)= vec(3)

C   ...Be sure out and back are opposite directions.
          IF (backv(nv,1)*outv(nv,1) .gt. 0.d0) THEN
            backv(nv,4)= -backv(nv,4)
            backv(nv,5)= -backv(nv,5)
            backv(nv,6)= -backv(nv,6)
          ENDIF
        ENDIF

C  Corrections (aberration and relativistic)
      ELSE IF (ephstr(1:2).eq.'30') THEN
        READ (ephstr(3:100),*,err=100) idir, (vec(k),k=1,3),corr

C    Outbound
        IF (idir .eq. 0 .OR. idir .eq. 1) THEN
          aberoutv(nv,1)= vec(1)
          aberoutv(nv,2)= vec(2)
          aberoutv(nv,3)= vec(3)
          c12(nv)= corr

C   ...In case there is no return information...
          aberbackv(nv,1)= -aberoutv(nv,1)
          aberbackv(nv,2)= -aberoutv(nv,2)
          aberbackv(nv,3)= -aberoutv(nv,3)
          c23(nv)= -c12(nv)

C    Inbound
        ELSE IF (idir .eq. 2) THEN
          aberbackv(nv,1)= vec(1)
          aberbackv(nv,2)= vec(2)
          aberbackv(nv,3)= vec(3)
          c23(nv)= corr

C  	WEAK TEST: REDO THIS!
          if (c23(nv)*c12(nv) .gt. 0.d0) c23(nv)= -c23(nv)
        ENDIF

C  Transponder
      ELSE IF (ephstr(1:2).eq.'40') THEN
        READ (ephstr(3:100),*,err=100) oscrelcorr

C  Offset from center of body
      ELSE IF (ephstr(1:2).eq.'50') THEN
        READ (ephstr(3:100),*,err=100) idir,mjd,sod,(vec(k),k=1,3)
        IF (idir .eq. 0 .OR. idir .eq. 1) nvoff= nvoff+ 1
        IF (nvoff.gt.MAXENT) THEN
          WRITE (*,*) 'ephemeris read error: too many records:',nvoff
          GO TO 110
        END IF
        offsetjdi(nvoff)=mjd+2400000.0d0
        offsetjdf(nvoff)=sod/86400.d0+0.5d0
        IF (offsetjdf(nvoff).ge.1.0d0) THEN
          offsetjdi(nvoff)=offsetjdi(nvoff)+1.0d0
          offsetjdf(nvoff)=offsetjdf(nvoff)-1.0d0
        ENDIF

C    Outbound or bounce
        IF (idir .eq. 0 .or. idir .eq. 1) THEN
          offsetoutv(nv,1)= vec(1)
          offsetoutv(nv,2)= vec(2)
          offsetoutv(nv,3)= vec(3)

C    Inbound
        ELSE IF (idir .eq. 2) THEN
          offsetbackv(nv,1)= vec(1)
          offsetbackv(nv,2)= vec(2)
          offsetbackv(nv,3)= vec(3)
        ENDIF

C  Rotation angle
      ELSE IF (ephstr(1:2).eq.'60') THEN
        nvrot= nvrot+ 1
        IF (nvrot.gt.MAXENT) THEN
          WRITE (*,*) 'ephemeris read error: too many records:',nvrot
          GO TO 110
        END IF
        READ (ephstr(3:100),*,err=100) mjd,sod,(rotangv(nvrot,k),k=1,3),
     x   gast(nvrot)
        rotjdi(nvrot)=mjd+2400000.0d0
        rotjdf(nvrot)=sod/86400.d0+0.5d0
        IF (rotjdf(nvrot).ge.1.0d0) THEN
          rotjdi(nvrot)=rotjdi(nvrot)+1.0d0
          rotjdf(nvrot)=rotjdf(nvrot)-1.0d0
        END IF
C       Sidereal time wraps around from 24 to 0 hours...
        gast(nvrot)= gast(nvrot)+ dgast
        if (nvrot .GT. 1 .AND. gast(nvrot) .LT. gast(nvrot-1)) then
           dgast= dgast+ 24.d0
           gast(nvrot)= gast(nvrot)+ 24.d0
        endif
        DO i=1,3
          rotangv(nvrot,i)= rotangv(nvrot,i)* PI/180.0d0
        ENDDO

C  EOP
      ELSE IF (ephstr(1:2).eq.'70') THEN
        nveop= nveop+ 1
        IF (nveop.gt.MAXENT) THEN
          WRITE (*,*) 'ephemeris read error: too many records:',nveop
          GO TO 110
        END IF
        READ (ephstr(3:100),*,err=100) mjd,sod,(eopxy(nveop,k),k=1,2), 
     x   eopdutc(nveop)
        eopjdi(nveop)=mjd+2400000.0d0
        eopjdf(nveop)=sod/86400.d0+0.5d0
        IF (eopjdf(nveop).ge.1.0d0) THEN
          eopjdi(nveop)=eopjdi(nveop)+1.0d0
          eopjdf(nveop)=eopjdf(nveop)-1.0d0
        END IF

      ELSE IF (ephstr(1:2).eq.'99') THEN  
!  Should do something here?
        GO TO 120

      ELSE IF (ephstr(1:1).eq.'#' .or. ephstr(1:2).eq.'00') THEN
!  Comment
      END IF
  
      GO TO 10
  
!     create a small usable time tag
  120 DO i=1,nv
        jds(i)=((jdf(i)-jdf(1))+(jdi(i)-jdi(1)))*86400.d0
      END DO
      DO i=1,nvoff
        offsetjds(i)=((offsetjdf(i)-offsetjdf(1))+
     &               (offsetjdi(i)-offsetjdi(1)))*86400.d0
      END DO
      DO i=1,nvrot
        rotjds(i)=((rotjdf(i)-rotjdf(1))+
     &               (rotjdi(i)-rotjdi(1)))*86400.d0
      END DO
      DO i=1,nveop
        eopjds(i)=((eopjdf(i)-eopjdf(1))+
     &               (eopjdi(i)-eopjdi(1)))*86400.d0
      END DO
      RETURN
  
  100 WRITE (*,*) 'ephemeris read error'
  110 CONTINUE
      ierr= -1
      if (debug) WRITE (*,*) 'nv=',nv
      RETURN
      END
