#include <stdio.h>
#include <math.h>
/* ========================================================================
CC  PURPOSE   : INTERPOLATE SATELLITE POSITION, CALCULATE SATELLITE VELOCITY 
CC              BASED ON CPF_INTER CODE FROM W. GURTNER
CC
CC  PARAMETERS:
CC         IN : TT       : EPOCH (TRANSMIT=FIRING TIME)            R*8
CC              IUNITS   : UNITS OF TIME (0 = SECONDS, #0 = DAYS)  I*4
CC                         (FOR TT AND TTAB!)
CC              TTAB     : TABULATED EPOCHS (SECONDS OR DAYS,      R*8
CC                                           SEE IUNITS)
CC              OBJTAB_X : TABULATED X-COORD. OF OBJECT (M)        R*8(*)
CC              OBJTAB_Y : TABULATED Y-COORD. OF OBJECT (M)        R*8(*)
CC              OBJTAB_Z : TABULATED Z-COORD. OF OBJECT (M)        R*8(*)
CC              NTAB     : NUMBER OF TABULATED EPOCHS/COORDINATES  I*4
CC              NINT     : NUMBER OF TABULATED VALUES TO USE       I*4
CC                         (ARGUMENT FOR LAGRANGE INTERPOLATION)
CC        OUT : X        : Geocentric X coordinate (M)             R*8
CC              Y        : Geocentric Y coordinate (M)             R*8
CC              Z        : Geocentric Z coordinate (M)             R*8
CC              VX       : Geocentric Velocity X compononet (M/S)  R*8
CC              VY       : Geocentric Velocity X compononet (M/S)  R*8
CC              VZ       : Geocentric Velocity X compononet (M/S)  R*8
CC              IRCODE   : RETURN CODE 0 = OK                      I*4
CC                                     1 = INTERPOLATION NOT IN
CC                                         CENTER INTERVAL
CC                                     2 = ERROR IN INTERPOLATION
CC
CC  AUTHOR:     S. RIEPL 
CC              BUNDESAMT FUER KARTOGRAPHIE UNFD GEODAESIE
CC              GERMANY
CC
CC  CREATED:    02-FEB-2006
CC
CC  MODIFIED:   
=========================================================================*/
void 
hermite (int ityp, double *x, double *y, double *z, int nmax, int nval, 
                double xp, double *yp, double *zp, int *ircode);
                
void
cpf_inter_irv(double tt,int iunits,double *ttab,double *objtab_x,
          double *objtab_y,double *objtab_z,int ntab,int nint,
          double *x,double *y,double *z,double *vx,double *vy,double *vz,
          int *ircode)
{
      double objtrt[3], dummy;
      int ircx, ircy, ircz;
      static int ifirst= 1;
      static double pi, prvlon, prvlat; 

      if (ifirst == 1) {

        pi=4.e0*atan(1.e0);
        prvlon=999.e0;
        prvlat=999.e0;
        ifirst=0;
      }

/*
 *  INSTANT VECTOR AT TRANSMIT TIME
 *  *******************************
 */
/*  Interpolate geocentric position of the object for transmit time */
    hermite(-1,ttab,objtab_x,&dummy,ntab,nint,tt,x,vx,&ircx);
    hermite(-1,ttab,objtab_y,&dummy,ntab,nint,tt,y,vy,&ircy);
    hermite(-1,ttab,objtab_z,&dummy,ntab,nint,tt,z,vz,&ircz);
    if (ircx+ircy+ircz != 0) {
        printf(" SR CPF_INTER: Return codes from SR HERMITE: %3i %3i %3i\n",
          ircx,ircy,ircz);
        *ircode=2;
        return;
    }
    *ircode=0;
/*    fprintf(stderr,"%15.7f %18.6f%18.6f%18.6f%18.6f%18.6f%18.6f\n",
        tt,*x,*y,*z,*vx,*vy,*vz);
*/
    return;
}
