#include "PredictionGenerator.h"

/******************************************************************************
Constructor
******************************************************************************/
CpfAllocator::CpfAllocator( YourTimeIntervalClass stepSize,
                             Cpf* cpf, EOP* eop, RotationAngles* rotAngles,
                             TargetOffsets* targetOffsets ) : PredictionGenerator(stepSize, cpf, eop)
{
    m_rotAngles = rotAngles;
    m_targetOffsets = targetOffsets;

    m_first = true;
    m_bidirectional = cpf->biDirectional();

    YourMatrix<double> posVelCor(7,3);
    m_posVelCor = posVelCor;
    YourMatrix<double> offsets(4,3);
    m_offsets = offsets;
    m_transponder.resize(5,0);
    m_accuracy.resize(9,0);
    setElement( cpf );
}

/******************************************************************************
Destructor
******************************************************************************/
CpfAllocator::~CpfAllocator( )
{
}

/******************************************************************************
******************************************************************************/
void CpfAllocator::setElement( Cpf* cpf )
{
    if( cpf->biDirectional() != m_bidirectional )
        yourTHROW( "Inconsistent direction flag between records.");

    m_cpfEpoch = cpf->epoch();

    m_posVelCor[0][0] = cpf->xPositionMm();
    m_posVelCor[0][1] = cpf->yPositionMm();
    m_posVelCor[0][2] = cpf->zPositionMm();
    m_posVelCor[1][0] = cpf->xPositionRxMm();
    m_posVelCor[1][1] = cpf->yPositionRxMm();
    m_posVelCor[1][2] = cpf->zPositionRxMm();
    m_posVelCor[2][0] = cpf->xVelocityMmPerDay();
    m_posVelCor[2][1] = cpf->yVelocityMmPerDay();
    m_posVelCor[2][2] = cpf->zVelocityMmPerDay();
    m_posVelCor[3][0] = cpf->xVelocityRxMmPerDay();
    m_posVelCor[3][1] = cpf->yVelocityRxMmPerDay();
    m_posVelCor[3][2] = cpf->zVelocityRxMmPerDay();
    m_posVelCor[4][0] = 1.0e-6 * cpf->xAberrationCorTx();
    m_posVelCor[4][1] = 1.0e-6 * cpf->yAberrationCorTx();
    m_posVelCor[4][2] = 1.0e-6 * cpf->zAberrationCorTx();
    m_posVelCor[5][0] = 1.0e-6 * cpf->xAberrationCorRx();
    m_posVelCor[5][1] = 1.0e-6 * cpf->yAberrationCorRx();
    m_posVelCor[5][2] = 1.0e-6 * cpf->zAberrationCorRx();
    m_posVelCor[6][0] = cpf->rangeRelCorRx();
    m_posVelCor[6][1] = cpf->rangeRelCorTx();
    m_posVelCor[6][2] = cpf->centreOfMassCorr();

    m_offsets[3][1] = cpf->rotationAngleType();
    m_offsets[3][2] = cpf->refFrame();

    // Build a consistent set of transponder data
    m_transponder[0] = cpf->transponder().pulseFrequency;
    m_transponder[1] = cpf->transponder().utcOffset;
    m_transponder[2] = cpf->transponder().transmitDelay;
    m_transponder[3] = cpf->transponder().oscillatorDrift;
    m_transponder[4] = cpf->transponder().oscillatorRelCor;

    //// Build a consistent set of accuracy data
    //Cpf::Accuracy* ro = cpf->accuracy();
    //if( ro )
    //{
    //    m_accuracy[0] =  ro->alongTrack[0];
    //    m_accuracy[1] =  ro->acrossTrack[0];
    //    m_accuracy[2] =  ro->radial[0];
    //    m_accuracy[3] =  ro->alongTrack[1];
    //    m_accuracy[4] =  ro->acrossTrack[1];
    //    m_accuracy[5] =  ro->radial[1];
    //    m_accuracy[6] =  ro->alongTrack[2];
    //    m_accuracy[7] =  ro->acrossTrack[2];
    //    m_accuracy[8] =  ro->radial[2];
    //}

    return;
}


/******************************************************************************
******************************************************************************/
void CpfAllocator::calculateNextPosition( YourTimeDateClass& epoch )
{
    m_epoch = m_epoch + m_stepSize;
    calculatePosition( m_epoch );
    epoch = m_epoch;
    return;
}

/******************************************************************************
******************************************************************************/
void CpfAllocator::calculatePosition( YourTimeDateClass epoch )
{
    if( m_first)
        m_epoch = m_cpfEpoch;
    else
    {
        if( fabs( YourTimeIntervalClass::seconds(epoch - m_cpfEpoch) ) > 0.01 )
            yourTHROW("Variation in time interval exceeds 10ms.");
    }
    m_first = false;


    // Build a consistent set of earth orentation parameters
    if( m_eop )
        m_eop->get( epoch, m_xPolar, m_yPolar, m_ut1_utc );

    // Build a consistent set of target offset and rotation angle data
    if( m_targetOffsets )
    {
        m_targetOffsets->getOut( epoch, m_offsets[0][0],
                                        m_offsets[0][1],
                                        m_offsets[0][2] );

        m_targetOffsets->getIn( epoch, m_offsets[1][0],
                                       m_offsets[1][1],
                                       m_offsets[1][2] );

        if( m_rotAngles )
        {
            m_rotAngles->get( epoch, m_offsets[2][0],
                                     m_offsets[2][1],
                                     m_offsets[2][2],
                                     m_offsets[3][0] );
        }
    }

	return;
}
