	PROGRAM perform_sighting
!******************************************************************************
!
!	This routine performs the actual sighting of the satellite from each
!	station. The routine also decides when a pass has started and when a
!	pass has completed.
!
!	PROGRAMMER & CREATION DATE
!	--------------------------
!		Mark S. Korenkiewicz		10/7/83
!
!	MODIFICATIONS
!	-------------
!	TO INCORPERATE THIS SOFTWARE INTO THE ON-SITE PACKAGE FOR
!	ACQUISITION.
!		BRION CONKLIN	03/15/90
!               PAUL J. SEERY   12/12/90
!		PAUL J. SEERY	04/08/92	START SEQUENCE
!		PAUL J. SEERY	04/16/92	MOD AZM ANGLES
!		PAUL J. SEERY   03/03/93        POLY FORMAT
!		R. L. Ricklefs	08/25/95	Properly handle
!				sun close and sun lit parameters
!		R. L. Ricklefs  08/14/06	sun info correction.
!
!******************************************************************************
	implicit none
	character*80	VERSION
        character*54    NTAB
	integer*2	SIC ,EPN
    	integer		i,IMIN_ELE,IDOY,IYEAR
	integer		cur_vec_p, F,IASND, isun(9), isuncl(9)
        INTEGER         MONTH ,DAY ,YEAR ,HOUR ,MIN, DOY, SEC
	real*8		win(9,12), curr_pass(25)
	real*8		min_ele, current_sat_elev, pca_elv, dtpca
	real*8		az

	logical		eof, pass_open, elv_up, sunlit, tooclose

	VERSION	=	"%W%	%G%"
	call getnames()
	do F = 1,25
	    curr_pass(F) = 0.0D0
	end do
	eof = .false.
	pass_open = .false.
	sunlit = .false.
	tooclose = .false.
	pca_elv = 0.0d0
	dtpca = 0.0d0
	IASND = 0
	READ(1,5) IMIN_ELE
    5	FORMAT(1X,I2)
	MIN_ELE = IMIN_ELE*1.0D0
	REWIND(1)

	READ(2,10) SIC,EPN,IDOY,IYEAR
   10	FORMAT(1X,I4,1X,I4,1X,i3,1x,i4)
	WRITE(3,10) SIC,EPN,IDOY,IYEAR

C
C	Look at the pass 8 time/position entries at a time, probably
C	since no on e knows a priori how many entries a pass will have
C
	DO F = 1,8
C
C	READ IN THE INERTIAL VECTOR FOR THIS TIME
C
		READ(2,20) (win(F,I),I=1,7)
   20	FORMAT(1X,F17.10,3(1X,F18.6),3(1X,F18.9))
C
C	READ IN THE AZ,ELE,RANGE,xp,yp
C
	READ(2,30) (win(F,i),i=8,12), isun(F), isuncl(F)
30	FORMAT(1X,F8.4,1X,F8.4,1X,F18.3,1x,f8.0,1x,f8.0,1x,i1,1x,i1)

	IF (F.NE.1) THEN
		CALL AZINFL(win(F,8),win(F-1,8))
	ENDIF
	ENDDO
1000	CONTINUE
	cur_vec_p = 5
	elv_up = .True.
C
C	READ IN THE BODY FIXED VECTOR FOR THIS TIME
C
40	CONTINUE

	READ(2,20,END=1150)(win(9,I),I=1,7)
C	READ IN THE AZ,ELE,RANGE,xp,yp
C
	READ(2,30,END=1150) (win(9,i),i=8,12), isun(9), isuncl(9)
	CALL AZINFL(win(9,8),win(8,8))
C
C  Check ascending arc
C	If already ascending...
	IF(IASND.EQ.1) GO TO 50
C	Jump is ascending...
	IF((win(1,9) - win(2,9)) .LT. 0.0D0) GO TO 35
C	Going back down...
C	  Toss oldest record from window
	    DO F =2,9
		DO I = 1,12
		    win(F-1,I) = win(F,I)
		ENDDO
		isun(F-1) = isun(F)
		isuncl(F-1) = isuncl(F)
	    ENDDO
	GO TO 40 
35	IASND = 1
50	CONTINUE
C	print *,'elv_up= ',elv_up
C	print *,'window= ',win
C	print *,'curr_pass= ',curr_pass

	call max_elevation (win, cur_vec_p, curr_pass, elv_up)

	current_sat_elev = win(cur_vec_p,9)

C	print *,'current_sat_elev= ',current_sat_elev
C	print *,'min_elv= ',min_ele
C	print *,'pass_open= ',pass_open
C	print *,'pca_elv= ',pca_elv
C	print *,'elv_up= ',elv_up

C	If still going up...
	IF (elv_up) THEN
	   IF (current_sat_elev.ge.min_ele) THEN

	    !**********************************************************
	    !we can see satellite and no pass is currently open for
	    !this station, therefore, initialize a pass for station
	    !**********************************************************
              If (.not.pass_open) call pass('AOS', curr_pass,
     *				win, MIN_ELE, 0.0d0, isun, isuncl)

	      PASS_OPEN = .TRUE.
	   ENDIF

	 ELSE IF ((pca_elv.EQ.0.0d0) .AND. (current_sat_elev.GE.
     x		MIN_ELE)) THEN
C	     Get PCA elevation...
	     call Parob(win, pca_elv, dtpca)
C	type 769, pca_elv
769	FORMAT(1X,f7.1)
	call pass('PCA', curr_pass, win, pca_elv, dtpca, isun, isuncl)
	    !
	    !**********************************************************
	    !we cannot see satellite if a pass is currently open for
	    !this station then complete the pass info and write the
	    !pass out to the ouput file, otherwise don't do anything
	    !**********************************************************
	 ELSE IF((current_sat_elev.LT.MIN_ELE) .AND. win(cur_vec_p-1,9)
     *		.GT. MIN_ELE) THEN
	    if (sunlit) curr_pass(23)= win(cur_vec_p,1)
	    if (tooclose) curr_pass(25)= win(cur_vec_p,1)
	    If (pass_open) call pass('LOS', curr_pass,win,
     * 					MIN_ELE, 0.0d0, isun, isuncl)
	    PASS_OPEN = .FALSE.
	    sunlit= .false.
	    tooclose= .false.
	    pca_elv = 0.0d0

		do F = 1,25
		    curr_pass(F) = 0.0D0
		end do
	ENDIF
C	Establish the start and stop time of sunlit and sun approach
	if (PASS_OPEN) then
	    if (.not.sunlit .and. isun(cur_vec_p).eq.1) then
		curr_pass(22)= win(cur_vec_p,1)
                if (cur_vec_p .gt. 1)
     x		  curr_pass(22)= win(cur_vec_p-1,1)
		sunlit= .true.
	    endif
	    if (sunlit .and. isun(cur_vec_p).eq.0) then
		curr_pass(23)= win(cur_vec_p,1)
                if (cur_vec_p .gt. 1)
     x		  curr_pass(23)= win(cur_vec_p-1,1)
		sunlit= .false.
	    endif
	    if (.not.tooclose .and. isuncl(cur_vec_p).eq.1) then
		curr_pass(24)= win(cur_vec_p,1)
		tooclose= .true.
	    endif
	    if (tooclose .and. isuncl(cur_vec_p).eq.0) then
		curr_pass(25)= win(cur_vec_p,1)
		tooclose= .false.
	    endif
	endif
C	Toss oldest record from window
	DO F = 2,9
	    DO I = 1,12
		win(F-1,I) = win(F,I)
	    ENDDO
	    isun(F-1) = isun(F)
	    isuncl(F-1) = isuncl(F)
	ENDDO

	GOTO 1000

	!**********************************************************************
	!if the above loop stopped because the last valid ephemeris point was
	!encountered or the end of the specified sighting interval was reached
	!then we have to close out all open passes
	!**********************************************************************
 1150	CONTINUE
	if (pass_open) then
	  if (sunlit) then
	     curr_pass(23)= win(cur_vec_p,1)
             if (cur_vec_p .gt. 1)
     x	        curr_pass(23)= win(cur_vec_p-1,1)
	  endif
	  if (tooclose) curr_pass(25)= win(cur_vec_p,1)
          call max_elevation (win, cur_vec_p, curr_pass, elv_up)
          call pass ('LOS',curr_pass, win, MIN_ELE, 0.0d0, isun, isuncl)
	end if
	end

!******************************************************************************
	subroutine max_elevation(win, cur_vec_p, curr_pass, elv_up)
!******************************************************************************
!	This routine tracks maximum elevation of the satellite. It stores the
!	largest value of elevation plus the values of range and azimuth at
!	maximum elevation in the last three elements of curr_pass. Therefore,
!	whenever a row of curr_pass is written out the last three elements 
!	should have the maximum elevation and range and azimuth at maximum
!	elevation for the pass.
!
!	ARGUMENTS
!	---------
!       elv_up      - set True for increasing elevation.
!                     set False at PCA.
!
!	curr_pass   - array that contains in each row the complete pass inform-
!		      ation for each station sighting is being done for
!		      (input).
!
!	cur_vec_p   - pointer into window that indicates the current satellite
!		      vector we are looking at (input).
!
!	win	    - array of 9 vectors, the 4 vectors before the current
!		      vector and 4 vectors after the current vector (input).
!
!	PROGRAMMER & CREATION DATE
!	--------------------------
!	Mark S. Korenkiewicz		10/7/83
!       Paul J. Seery                   12/3/90
!	Paul J. Seery			03/02/93	Poly Format
!
!	MODIFICATIONS
!	-------------
!******************************************************************************
	implicit none

	integer		cur_vec_p

	real*8		curr_pass(25), win(9,12)
	real*8		azimuth, elevation, range

	logical		elv_up

	azimuth   = win(cur_vec_p, 8)
	elevation = win(cur_vec_p, 9)
	range	  = win(cur_vec_p, 10)

C	print *,'elevation= ',elevation
C	print *,'curr_pass(10)= ',curr_pass(10)

	if (elevation.gt.curr_pass(11)) then
	    curr_pass(10) = azimuth		! PCA positions
	    curr_pass(11) = elevation
	    curr_pass(12) = range
		curr_pass(19) = win(cur_vec_p,11)	! X-pole
		curr_pass(20) = win(cur_vec_p,12)	! Y-pole
	else
	   elv_up = .False.
	end if

	return
	end

!******************************************************************************
	subroutine pass(func, curr_pass, win,site_elv,dtpca,isun,isuncl)
!******************************************************************************
!	This routine stores pass information from the window array into the
!	curr_pass array. It decides what and where to store information by the
!	value of func which can be either 'AOS', 'PCA', or 'LOS'. These indicat-
!	ors tell pass whether it is to store information for the beginning of a
!	pass or completion of a pass. If a pass is completed the information
!	is stored in curr_pass then written out to the output file and then the
!	row of curr_pass for the station is cleared.
!
!	ARGUMENTS
!	---------
!	func	    - indicates the function the subroutine is to perform. If
!		      func = 'AOS' then the routine stores information that
!		      pertains to the beginning of a pass. If func = 'PCA'
!                     then the subroutine stores information pertaining to the
!                     apex of the pass. If func = 'LOS'
!		      then the subroutine stores information pertaining to the
!		      completion of a pass (input).
!
!	curr_pass   - array that contains in each row the complete pass inform-
!		      ation for each station sighting is being done for
!		      (input).
!
!	win	    - array of 9 vectors, the 4 vectors before the current
!		      vector and 4 vectors after the current vector (input).
!
!	PROGRAMMER & CREATION DATE
!	--------------------------
!		Mark S. Korenkiewicz		10/7/83
!
!	MODIFICATIONS
!               Paul J. Seery                   12/07/90
!		Paul J. Seery			04/15/93
!			included sun lit output
!		R. L. Ricklefs			08/25/95
!			include sun close output
!	-------------
!******************************************************************************
	implicit none

	integer 	i, F, isun(9), isuncl(9)

	real*8		curr_pass(25), win(9,12), site_elv, dtpca

	character*3	func

	if (func.eq.'AOS') then
	    !*********************
	    !initialize pass array
	    !*********************
	curr_pass(21) = 1.	 !set flag showing a pass is in
				 !progress

	    call store (func,curr_pass, win, site_elv,dtpca)

	else if (func.eq.'PCA') then
	    !*************************************************************
	    !complete pass array by saving ending information then writing
	    !out complete pas information to output file
	    !*************************************************************
	  curr_pass(21) = 2.     ! set for PCA
	  call store (func,curr_pass, win, site_elv,dtpca)
	else
	  call store(func,curr_pass,win, site_elv,dtpca)
		do F = 2,10,4
			if(curr_pass(F).lt.0.0d0) then
			curr_pass(F)=curr_pass(F)+ 360.0d0
			else
			if(curr_pass(F).ge. 360.0d0) then
			curr_pass(F)= curr_pass(F) - 360.0d0
			endif
			endif
       		enddo

		WRITE(3,100)(curr_pass(i), i=1,25)
 100		format(3(f18.10, 2f12.6, f18.6), 3f18.7, 3f18.9, 
     x			2f8.1, f4.1, 4f18.10)

	do F = 1,25
	    curr_pass(F) = 0.0D0	!everything gets cleared
	end do

	end if

	return
	end

!******************************************************************************
	subroutine store (func,curr_pass,win,site_elv,dtpca)
!******************************************************************************
!	This routine knowing the function to be performed (i.e. - either init-
!	ialize a pass or complete a pass) interpolates the values found in
!	the window array and stores these values in the curr_pass array.
!
!	ARGUMENTS
!	---------
!	func	    - indicates the function the subroutine is to perform. If
!		      func = 'AOS' then the routine stores information that
!		      pertains to the beginning of a pass. If func= 'PCA or LOS'
!		      then the subroutine stores information pertaining to the
!		      completion of a pass (input).
!
!	curr_pass   - array that contains in each row the complete pass inform-
!		      ation for each station sighting is being done for
!		      (input, output).
!
!	win	    - array of 9 vectors, the 4 vectors before the current
!		      vector and 4 vectors after the current vector (input).
!******************************************************************************
	implicit none

	integer		i, j, offset

	real*8		curr_pass(25), win(9,12)
	real*8		x(8), x_to_find, f_of_x(8), site_elv, dtpca
	real*8		result ,rusl,interpolate, slope, b ,SOD

	character*3	func

	DO I = 1,8
		X(I) = 0.0D0
		F_OF_X(I) = 0.0D0
	ENDDO
	if (func.eq.'AOS') then		        !offset indicates which set of
	    offset = 0				!of values AOS or LOS gets 
	  else if (func.eq.'PCA') then
	    offset = 8
	  else			
	    offset = 4
	end if 

	x_to_find = site_elv   !all interpolations return values at the
			       ! sighting elevation
	
	do i = 1,8		       !store elevations for window vectors x
	    x(i) = win(i,9)	       !is the same for all interpolations
	end do

	!**********************************************************************
	!if we initialize or complete a pass after/before the satellite reaches
	!the minimum sighting elevation we must adjust the x_to_find so that it
	!is just the current vector
	!**********************************************************************
	if (func.eq.'AOS') then
	    if (.not.((x(4).lt.x_to_find).and.(x_to_find.lt.x(5))))
     *                                           x_to_find = x(5)
	else if (func.eq.'LOS') then
	    if (.not.((x(4).gt.x_to_find).and.(x_to_find.gt.x(5))))
     *                                           x_to_find = x(5)
	end if

	!**********************************************************************
	!setup to interpolate MJD date this interpolation has to be treated
        ! as a special case due to the possibility that a the day may change.
	!**********************************************************************

	do i = 8,1,-1
		f_of_x(i) = win(i,1)
	end do

	!**********************************************************************
	!due to the fact that time is not a function of elevation we must
	!calculate the slope between x(4) and x(5) and straight line points
	!x(6,7,8) if we are initializeing a pass or x(1,2,3) if we are com-
	!leteing a pass. This allows the interpolation to be well behaved
	!especially for low, fast satellites on very short, steep passes.
	!**********************************************************************
	if (func.eq.'AOS') then
	    slope = (f_of_x(5) - f_of_x(4)) / (x(5) - x(4))
	    b = f_of_x(4)

	    do i = 6,8
		x(i) = (f_of_x(i) - b) / slope + x(4)
	    end do
	else if (func.eq.'LOS') then
	    slope = (f_of_x(4) - f_of_x(5)) / (x(4) - x(5))
	    b = f_of_x(5)

	    do i = 1,3
		x(i) = (f_of_x(i) - b) / slope + x(5)
	    end do
	end if

	if (func.eq.'PCA') then
	result = dtpca + win(3,1)
C	type 69, dtpca
69 	format (1x,f12.4)
	else
	result = interpolate (x, f_of_x, x_to_find)
	endif
	rusl = RESULT	
	SOD =IDNINT(DMOD(RESULT,1.0D0)*86400.0D0)
	RESULT = IDINT(RESULT) + SOD/86400.0D0
CC	SOD =JIDNNT(DMOD(RESULT,1.0D0)*86400.0D0)
CC	RESULT = JIDINT(RESULT) + SOD/86400.0D0
C	print *,rusl,SOD,RESULT
	
	
	    curr_pass((1 + offset)) = RESULT

	!**********************************************************************
	!the rest of the interpolations for values for AOS at MSE are straight
	!forward interpolations we replace the elevations is x with seconds of
	!day since all the following functions are with respect to time.
	!**********************************************************************
	do J = 1,8
	    x(J) = win(J,1)
	end do
	x_to_find = result

	do i = (2 + offset), (4 + offset)
	    do j = 1,8
	        f_of_x(j) = win(J,(I-OFFSET+6))
	    end do
	
	    curr_pass(i) = interpolate (x, f_of_x, x_to_find)

	end do

	if(offset .eq. 8) then
	do i = (5 + offset), (10 + offset)
	    do j = 1,8
	        f_of_x(j) = win(J,(I-OFFSET-3))
	    end do
	
	    curr_pass(i) = interpolate (x, f_of_x, x_to_find)

	end do
	endif

	return
	end

!******************************************************************************
	real*8 function interpolate(x, f_of_x, x_to_find)
!******************************************************************************
!	This routine performs an 8-point Lagrange interpolation.
!
!	ARGUMENTS
!	---------
!	x	  = array of 8 known x values.
!
!	f_of_x    = array of 8 known function values corresponding to x values.
!
!	x_to_find = value of x for which we need the function value .
!
!	PROGRAMMER & CREATION DATE
!	--------------------------
!		Win Decker		10/6/83
!
!	MODIFICATIONS
!	-------------
!******************************************************************************
        implicit none

	integer		i, j

	real*8		x(8), f_of_x(8), x_to_find
	real*8		a(8), n(8), d(8), atotal, fxout1, fxout2

	do i = 1,8
	    if (x_to_find.eq.x(i)) then
		interpolate = f_of_x(i)
		return
	    end if
	end do

	do i = 1,8
	    a(i) = x_to_find - x(i)
	end do

        atotal = a(1)*a(2)*a(3)*a(4)*a(5)*a(6)*a(7)*a(8)
	do i = 1,8
	    n(i) = atotal / a(i)
	end do

	do i = 1,8
	    d(i) = 1.0D0
	    do j = 1,8
		if (i.ne.j) d(i) = d(i) * (x(i) - x(j))
	    end do
	end do

	fxout1 = 0.0D0
	fxout2 = 0.0D0

	fxout1 = (n(1)/d(1)) * f_of_x(1) + (n(2)/d(2)) * f_of_x(2) + 
     *           (n(3)/d(3)) * f_of_x(3) + (n(4)/d(4)) * f_of_x(4)

	fxout2 = (n(5)/d(5)) * f_of_x(5) + (n(6)/d(6)) * f_of_x(6) +
     *           (n(7)/d(7)) * f_of_x(7) + (n(8)/d(8)) * f_of_x(8)

	interpolate = fxout1 + fxout2

	return
	end
!
	subroutine Parob(win, pca_elv, dt)
	implicit real*8 (a-h,o-z)
	dimension win(9,10)

	t1 = win(4,1) - win(3,1)
	t2 = win(5,1) - win(3,1)
	t1s = t1*t1
	t2s = t2*t2 

	d1 = win(4,9) - win(3,9)
	d2 = win(5,9) - win(3,9)

	b = (t1s*d2-t2s*d1)/(t2*t1s-t1*t2s)
	a = (d2-t2*b)/t2s
	dt = -b/(2.0*a)
C	print *,'dt= ',dt
	pca_elv = a*dt*dt + b*dt + win(3,9)
	return
	end
	
	subroutine azinfl(az2, az1)
	real*8 az2, az1, daz

	daz = az2 - az1
	IF(DABS(daz) .gt. 180.0d0) then
		IF(daz .lt. 0.0d0) then
			az2 = az2 + 360.0d0
		ELSE
			az2 = az2 - 360.0d0
		ENDIF
	ENDIF

	RETURN
	END

	SUBROUTINE DATETIME_OLD(MJD,YEAR,DOY,MONTH,DAY,HOUR,MIN,SEC)
C***********************************************************************
C
C
C	THIS CONVERTS FROM MODIFIED JULIAN DAY TO STANDARD
C	YEAR,MONTH,DAY FORMAT
C
C	Modified for years		Paul J. Seery	03/03/93
C	Fixed for incorrect JD		R. L. Ricklefs	01/xx/94
C
C***********************************************************************

	INTEGER YEAR,DOY,MONTH,DAY,HOUR,MIN,SEC,JD
	INTEGER I,N,MTH(12)
	REAL*8 MJD,FOY,FOD,TSEC,FJD

	DATA MTH/0,31,28,31,30,31,30,31,31,30,31,30/

	JD = MJD
	FOD = MJD - JD
	TSEC = FOD * 86400.0D0
	HOUR = TSEC/3600
	MIN = (TSEC - HOUR*3600)/60
	SEC = TSEC - HOUR*3600 - MIN*60 + 0.05

	FJD = JD - 15018
	YEAR = FJD/365.25
	FOY = FJD - YEAR*365.25
CC Y2K	YEAR = YEAR + 1900
        IF (YEAR .LE. 60) THEN
	    YEAR= YEAR+ 2000
        ELSE
	    YEAR= YEAR+ 1900
        ENDIF
	DOY = FOY- 1
	IF(((FOY-DOY+0.05) - 0.25) .GE. 0.0) DOY = DOY + 1

	MTH(3) = 28
	N = YEAR/4
	if((YEAR - N*4) .eq. 0) MTH(3) = 29 
	DAY = DOY
	DO 10 I=1,12
	IF((DAY - MTH(I)) .LE. 0) GO TO 20
	DAY = DAY - MTH(I)
	MONTH = I
10	CONTINUE
20	CONTINUE
	RETURN
	END

      SUBROUTINE DATETIME(MJD,YEAR,DOY,MONTH,DAY,HOUR,MINUTE,SEC)
C***********************************************************************
C
C
C       THIS CONVERTS FROM MODIFIED JULIAN DAY TO STANDARD
C       YEAR,MONTH,DAY FORMAT
C
C       Modified for years              Paul J. Seery   03/03/93
C       Fixed for incorrect JD          R. L. Ricklefs  01/xx/94
C       Totally replaced code due to error converting for day 12/31.
C                                       R. L. Ricklefs  12/08/03.
C
C***********************************************************************
      REAL*8 MJD
      DOUBLE PRECISION JDINT,JDF,JDA1900,JDFC, SECOND
      DOUBLE PRECISION JD_IN, JD_JAN0
      INTEGER YEAR,MONTH,DAY,HOUR,MINUTE,SEC, DOY, TDAY
C
C      JDA1900 = DINT(JDINT) - 2415079.0D0
      JDA1900 = DINT(mjd) - 15078.0D0
      JDFC= MJD- DINT(MJD)
 
 5    HOUR   = IDINT(JDFC*24.D0)
      MINUTE = IDINT((JDFC - HOUR/24.D0)*1440.D0)
      SECOND = (JDFC - HOUR/24.D0 - MINUTE/1440.D0)*86400.D0
      SEC    = SECOND+ 0.05
      YEAR   = IDINT(((4.D0*JDA1900)-1)/1461.D0)
 
      TDAY   = DINT(((4.D0*JDA1900)+3.D0- (YEAR*1461.D0))/4.D0)
      MONTH  = IDINT(((5.D0*TDAY)-3.D0)/153.D0)
      DAY    = IDINT(((5.D0*TDAY)+2.D0 - (153.D0*MONTH))/5.D0)
 
      IF(MONTH.LT.10) GO TO 10
         MONTH = MONTH - 9
         YEAR  = YEAR + 1
         GO TO 99
 10      MONTH = MONTH + 3
 99   CONTINUE
 
C Now get Day of Year
C  calculate # days since noon feburary 29, 1900 (julian date=2415078.0)
      if (month .LE. 2) THEN
        jd_in = idint(1461.0d0 * (year-1)/4.0d0)
     x            + idint((153.0d0 * (month+9) + 2.0d0)/5.0d0) + day
 
      ELSE
        jd_in = idint(1461.0d0 * year/4.0d0)
     x           + idint((153.0d0 * (month-3) + 2.0d0)/5.0d0) + day
      ENDIF 

C  what is jd of jan 0?
      jd_jan0 = idint(1461.0d0 * (year-1)/4.0d0)+ 306.0d0
 
C  now doy is easy. 
      doy= jd_in- jd_jan0
      year= year+ 1900
      END
