C*
      PROGRAM CPF_PGM
CC
CC NAME       :  CPF_PGM
CC
CC PURPOSE    :  COMPUTES AZIMUTH/ELEVATION/RANGE VALUES FOR A SATELLITE PASS
CC               USING CPF GEOCENTRIC SATELLITE POSITIONS
CC
CC               SAMPLE PROGRAM
CC
CC SR CALLED  :  CPF_INTER (CALLING HERMITE, DDREH, DMLMAV)
CC               EPHINITU
CC
CC INCLUDES   :  EPHI.INC, EPHIHDR.INC
CC
CC REMARKS    :  STATION COORDINATES ARE "HARDWIRED" INTO THE PROGRAM
CC
CC               THE PROGRAM DOES NOT HANDLE LEAP SECOND TRANSITIONS!
CC
CC AUTHOR     :  W. GURTNER
CC
CC
CC CREATED    :  12-OCT-2005
CC
CC CHANGES    :  29-NOV-2005 (WG) : CENTER OF MASS CORRECTION
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      2005      UNIVERSITY OF BERNE
CC                    SWITZERLAND
C*

      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER CPFFIL*40,FILEPH*40
      REAL*8    STAXYZ(3)
C
C      INCLUDE  'EPHI.INC'
C      INCLUDE  'EPHIHDR.INC'
      INCLUDE  '../include/ephi.inc'
      INCLUDE  '../include/ephihdr.inc'
C
      COMMON /deb/ debug
      LOGICAL debug
C
C STATION COORDINATES (XYZ, LATITUDE, LONGITUDE EAST>0)
      DATA STAXYZ /4331283.557D0,567549.902D0,4633140.353D0/
      DATA STALAT /46.877229D0/
      DATA STALON /7.4652415D0/
C
C NUMBER OF VALUES TO BE USED FOR INTERPOLATION (SR HERMITE)
      DATA NVAL/10/
C
C EPHINITU: DEBUG NO
      debug=.FALSE.
C
      PI=4.D0*DATAN(1.D0)
C RADIANS
      STALAT=STALAT*PI/180.D0
      STALON=STALON*PI/180.D0
C
      LFNCPF=20
      LFNEPH=30
C
C Get Input Options from Input Queue
C ----------------------------------
C
C Filename of CPF prediction input file
      WRITE(*,*) 'CPF prediction file?'
      READ(*,1) CPFFIL
1     FORMAT(A)
C Filename of ephemeris output file
      WRITE(*,*) 'Output file?'
      READ(*,1) FILEPH
C Start time (Modified Julian Date)
      WRITE(*,*) 'Start time (Modified Julian Date)?'
      READ(*,*) TSTART
C End time (MJD)
      WRITE(*,*) 'Stop  time (Modified Julian Date)?'
      READ(*,*) TEND
C Increment (seconds)
      WRITE(*,*) 'Time increment (seconds)?'
      READ(*,*) DT
C Time Bias (seconds)
      WRITE(*,*) 'A priori time bias (sec, will be added to epochs)?'
      READ(*,*) TBIAS
C
C  Read CPF File
C  -------------
C
C  JDI,JDF,JDS,OUTV,NV USED BELOW ARE STORED BY SUBR EPHINITU INTO COMMON
C  (EPHEMERIDES: EPOCH, X,Y,Z, NUMBER OF VECTORS)
C  (SEE INCLUDE FILES)
C
      OPEN(UNIT=LFNCPF,FILE=CPFFIL,STATUS='OLD',IOSTAT=IOS)
      IF(IOS.NE.0) THEN
        WRITE(*,*) '*** PGM CPF_PGM: Error opening CPF file ',CPFFIL
        GOTO 999
      END IF
      CALL EPHINITU (LFNCPF,IERR)
      CLOSE(UNIT=LFNCPF)
      IF(IERR.NE.0) THEN
        WRITE(*,*) 'PGM CPF_PGM: Error reading CPF file ',CPFFIL
        GOTO 999
      END IF
C
C Is this an slr target or llr/tlr?
      IF(ltcorr.EQ.1.OR.tar_type.EQ.2.OR.compat.NE.1) THEN
        WRITE(*,*) 'PGM CPF_PGM: File ',CPFFIL
        write(*,*) '             requires LLR/TLR prediction software!'
        GOTO 999
      ENDIF
C
C TOPOCENTRIC VECTORS FOR ALL EPOCHS, FLIGHT TIME APPLIED
C *******************************************************
C
C OUTBOUND VECTOR, POSITION OF SATELLITE AT BOUNCE TIME
C AZIMUTH AND ELEVATION FOR LASER BEAM AT TRANSMIT TIME
C AZIMUTH AND ELEVATION FOR RECEIVER AT TRANSMIT TIME: ADD DIFAZI/DIFELE
      MODE=1
C TIME UNITS (FOR JDS, SEE SR EPHINITU): SECONDS
      IUNITS=0
C
C FILE TO STORE THE INTERPOLATED VALUES
      OPEN(UNIT=LFNEPH,FILE=FILEPH,STATUS='UNKNOWN')
C
C MODIFIED JULIAN DATE
      I=0
110     TJUL=TSTART+I*DT/86400.D0
        IF(TJUL.GT.TEND) GOTO 120
C RELATIVE TIME TO START OF CPF FILE
C   JDI (FULL DAYS), JDI (FRACTION) GIVEN IN JULIAN DATE
C   JDS: DAYS RELATIVE TO FIRST EPOCH
        TIM=(TJUL-(JDI(1)+JDF(1)-2400000.5D0))*86400.D0+TBIAS
C
C OUTV(1,1),OUTV(1,2),OUTV(1,3): X,Y,Z
C
        CALL CPF_INTER(TIM,IUNITS,MODE,STAXYZ,STALAT,STALON,COFM_CORR,
     1                 JDS,OUTV(1,1),OUTV(1,2),OUTV(1,3),NV,NVAL,
     2                 RANGE,FLTIME,AZI,ELE,DIFAZI,DIFELE,IRC)
        WRITE(LFNEPH,1000) TJUL,AZI,ELE,DIFAZI,DIFELE,FLTIME
 1000   FORMAT (F11.5,2(2X,F10.5),2(2X,F8.5),2X,F13.10)
        I=I+1
        GOTO 110
120   CLOSE(UNIT=LFNEPH)
C
999   STOP
      END
